<?php
/**
 * AutoSOAP - Expanded SOAP Server
 * 
 * PHP version 5
 * 
 * @package jp.servlet.AutoSOAP
 * @author Sakamoto Kouichi <sakamoto@servlet.sakura.ne.jp> 
 * @copyright 2006 Sakamoto Kouichi
 * @license http://www.apache.org/licenses/LICENSE-2.0 Apache Software License 2.0
 * 
 * $Id: Framework.class.php 87 2006-04-16 00:39:49Z sakamoto $
 */

include_once(dirname(__FILE__)."/AutoSOAP.php");
include_once(dirname(__FILE__)."/ServiceFactory.class.php");

/**
 * AutoSOAPt[[N
 * 
 * @package jp.servlet.AutoSOAP
 * @author Sakamoto Kouichi <sakamoto@servlet.sakura.ne.jp> 
 */
class AutoSOAP_Framework {
    /**
     * T[oCX^X
     * @var AutoSOAP_Server
     */
    private $_server;
    
    /**
     * T[rXt@Ng[
     * @var array
     */
    private $_factories;
    
    /**
     * x[XfBNg[
     * @var string
     */
    private $base_dir;
    
    /**
     * CX^XIuWFNg
     * @var AutoSOAP_Framework
     */
    private static $instance;

    /**
     * CX^̎擾
     * 
     * @access public 
     * @return AutoSOAP_Framework
     */
    static public function getInstance($base_dir=null,$without_xsd=false)
    {
        if (is_null(self :: $instance) && !is_null($base_dir)) {
            $class = __CLASS__;
            self :: $instance = new $class($base_dir,$without_xsd);
        } 
        return self :: $instance;
    } 

    /**
     * RXgN^
     * @param string $base_dir
     */
    private function __construct($base_dir,$without_xsd)
    {
        /*
         * ݒ
         */
        $this->_factories = array();
        $this->base_dir = (is_null($base_dir)) ? dirname(__FILE__).'/' : $base_dir.'/' ;
        
        $services_dir1  = $this->base_dir.'services/';
        $services_dir2  = dirname(dirname(__FILE__)).'/';
        $wsdls_dir      = $this->base_dir.'wsdls/';
        $xsds_dir       = ($without_xsd) ? null : $this->base_dir.'xsds/';
        $classmaps_dir  = $this->base_dir.'classmaps/';
        
        $this->_factories['custom'] = new AutoSOAP_ServiceFactory(
                            $services_dir1,
                            $wsdls_dir,
                            $xsds_dir,
                            $classmaps_dir
                       );

        $this->_factories['autosoap'] = new AutoSOAP_ServiceFactory(
                            $services_dir2,
                            $wsdls_dir,
                            $xsds_dir,
                            $classmaps_dir
                       );
        /*
         * SOAPT[o
         */
        $this->_server = AutoSOAP_Server::getInstance();
        $this->_server->setEncoding("UTF-8", "Shift_JIS");
    }
    
    /**
     * SOAPT[rX̎s
     * @param string $location
     * @param string $service_name
     */
    public function execute($alias=null, $location=null, $service_name=null)
    {
        $factory = 'custom';
        if (is_null($service_name)) {
            $service_name = $_SERVER["QUERY_STRING"];
        }
        if (is_null($location)) {
            $flg_https = false;
            if(isset($_SERVER["HTTPS"])) {
                $flg_https = ('on'==$_SERVER["HTTPS"]);
            }
            $location = (($flg_https)?"https://":"http://") . $_SERVER["SERVER_NAME"] . $_SERVER["SCRIPT_NAME"] . '?';
        }
        
        $wsdl_mode    = false;
        $matches      = array();
        if (preg_match("/^(.+)@wsdl$/", $service_name, $matches)) {
            $service_name = $matches[1];
            $wsdl_mode = true;
        }
        
        $location_query = $service_name;
        
        // GCAXKp
        if (is_array($alias)) {
            foreach ($alias as $pattern => $replace) {
                if ('/'===substr($pattern,0,1)) {
                    if (preg_match($pattern, $service_name)) {
                        $service_name = preg_replace($pattern, $replace, $service_name);
                        break;    
                    } else if ($pattern===$service_name) {
                        $service_name = $replace;
                    }
                }
            }
        }
        if (0==strlen($service_name)) {
            trigger_error("Service was not found.",E_USER_ERROR);
            return;
        }
        
        //AutoSOAP̃Cu[T[rX񋟂
        if (preg_match("/^autosoap\.services\./i",$service_name)) {
            $factory = 'autosoap';
        }
        
        /*
         * T[rX
         */
        $service = $this->_factories[$factory]->getService($service_name, ($location . $service_name));
        
        /*
         * SOAPT[rXT[o֐ݒ肷
         */
        $this->_server->setService($service);
        
        if ($wsdl_mode) {
            /*
             * WSDLԂ
             */
            $this->_server->wsdl();
        } else {
            /*
             * SOAPT[rXs
             */
            $this->_server->execute();
        }
    }    

    /**
     * MONXZbg
     * 
     * @access public 
     * @param AutoSOAP_Logger $logger
     * @return boolean 
     */
    public function setLogger(AutoSOAP_Logger $logger)
    {
        $this->_server->setLogger($logger);
    } 

    /**
     * tB^[ǉ
     * @param string $filter_name
     */
    public function appendFilter($filter_name)
    {
        include_once($this->base_dir.'filters/'.$filter_name.".filter.php");
        $this->_server->appendFilter(new $filter_name());
    }

    /**
     * PeartB^[ǉ
     * @param string $filter_name
     */
    public function appendPearFilter($filter_name)
    {
        include_once(dirname(__FILE__).'Framework/filters/'.$filter_name.".filter.php");
        $this->_server->appendFilter(new $filter_name());
    }

    /**
     * f[^ۑfBNg[Ԃ
     */
    public function getVarDir()
    {
        return $this->base_dir."vars/";
    }
    
    /**
     * ZbgAbv
     */
    public function setup()
    {
        restore_error_handler();
        $flg_succeed = true;
        $dirs = array(
                    '',
                    'services',
                    'wsdls',
                    'xsds',
                    'classmaps',
                    'filters',
                    'vars',
        );
        echo "<font color=blue><b>Start AutoSOAP Framework Setup...</b></font>"."<br>\n";
        foreach ($dirs as $name) {
            $dir = $this->base_dir.$name.'/';
            $mssg_dir = '"'.$dir.'"';
            $name = (''===$name) ? 'Base Directory' : "dir:{$name}";

            echo "check {$name}...";
            if (true === file_exists($dir)) {
                if (is_file($dir)) {
                    $flg_succeed = false;
                    echo "<br>\n&nbsp;&nbsp;&nbsp;";
                    echo "<font color=red>This is file. Please make directory.</font>";
                } else {
                    echo "ok";
                }
            } else {
                echo "<font color=blue>create...</font>";
                if (false === @mkdir($dir)) {
                    $flg_succeed = false;
                    echo "<br>\n&nbsp;&nbsp;&nbsp;";
                    echo "<font color=red>The directory was not able to be made. Please change the permission of \"{$this->base_dir}\" to 777.</font>";
                } else {
                    echo "ok";
                }
            }
            echo "<br>\n";
        }
        echo "<br>\n";
        if ($flg_succeed) {
            echo "<font color=blue><b>The setup of AutoSOAP Framework succeeded. </b></font>"."<br>\n";
        } else {
            echo "<font color=red><b>The setup of AutoSOAP Framework failed. </b></font>"."<br>\n";
        }
        exit;
    }
}
?>